function [Y_cycle, Y_trend] = HP_FILTER(Y,lambda,plot_flag)

% Purpose: 
% Decompose a series into the cyclical component and the trend component
% with Hodrick-Prescott filter
% -----------------------------------
% Model:
% Y(t) = Y_cycle(t) + Y_trend(t)
% The trend term minimizes
% sum{ [Y(t) C Y_trend(t)]^2} + lambda * sum{[Y_trend(t+1) C 2*Y_trend(t) + Y_trend(t-1)]^2}
% where lambda is a smooth parameter.
% For monthly data, suggested usage: lambda = 14400 or 129600
% For quarterly data, suggested usage: lambda = 1600
% For annual data, suggested usage: lambda = 100 or 6.5
% -----------------------------------
% Usage:
% Y = the original series, T-by-1 vector. (Multiple series are supported, T-by-k matrix)
% lambda = the smooth parameter.
% plot_flag: 0 = no plot; 1 = plot original, trend and detrended series in a graph
% -----------------------------------
% Returns:
% Y_cycle = the cyclical component of the series (a new detrended series)
% Y_trend = the trend component of the series
% -----------------------------------
% Written by Hang Qian, Iowa State University
% Contact me:  matlabist@gmail.com


if nargin < 2;  lambda = 1600;   end
if nargin < 3;  plot_flag = 0;   end

% Analyze data
[nobs, nvar] = size(Y);
if nobs < nvar
    Y = Y';
    [nobs, nvar] = size(Y);
end
nobs2 = nobs - 2;

% Decompose the series into the cyclical component and trend component
if nobs <= 100
    % For matrix of smaller dimension, apply the formula:  inv(I+K'*K)*Y
    seq = [1;-2;1];
    K = zeros(nobs,nobs2);
    for t = 1:nobs2
        K(t:t+2,t) = seq;
    end
    A = eye(nobs) + lambda .* (K * K');
    Y_trend = A \ Y;
    Y_cycle = Y - Y_trend;
else
    % For matrix of larger dimension, use sparse matrix
    col_ind = 1:nobs2;
    col_ind = col_ind(ones(3,1),:);
    row_ind = col_ind;    
    row_ind(2,:) = row_ind(2,:) + 1;
    row_ind(3,:) = row_ind(3,:) + 2;
    seq = [1;-2;1];
    seq = seq(:,ones(nobs2,1));
    K = sparse(row_ind(:), col_ind(:), seq(:), nobs, nobs2, 3*nobs2);
    A = sparse(1:nobs,1:nobs,1,nobs,nobs,nobs) + lambda .* (K * K');
    Y_trend = A \ Y;
    Y_cycle = Y - Y_trend;
end

if plot_flag == 1    
    for m = 1:nvar
        figure(m)
        seq = (1:nobs)';
        plot(seq,Y(:,m),'-b', seq,Y_trend(:,m),':r', seq, Y_cycle(:,m),'--m');
        legend('Series','Trend','Cycle');
        title('Decompose a Series by Hodrick-Prescott Filter')
    end
end
        
