function [Phi_draws, Sigma_draws,constant_draws] = VARDAS_SIMPLE(Y,nlag,sum_flag,add_constant,ndraws,burn_in)

% Purpose: 
% Bayesian inference of mixed frequency VAR(p) model (VARDAS)
% This version only support data of two frequencies, 
% and a series cannot change frequency halfway
% -----------------------------------
% Model:
% The latent Y(t) contains d component variables and follows VAR(p)
% Y(t) = c + phi(1) * Y(t-1) + ...+ phi(p) * Y(t-p) + ui,
% where ui ~ N(0,Sigma)
% However, some of the Y(t) are aggregated or averaged (See notes below)
% -----------------------------------
% Algorithm: 
% Qian (2011) "Bayesian VAR with varied frequency data"
% Conditional on latent data, this is largely a linear regression model.
% Conditional posteriors are conjugate normal and inverted gamma respectively.
% Latent variables with aggregation constraints are sampled all at once 
% from conditional MVN by transforming the covariance matrix of NT observations
% -----------------------------------
% Usage:
% Y = variables in VAR (T * d matrix, or d * T matrix)
% nlag = number of lags (p) in the VAR model
% sum_flag: specify how the latent high frequency data are grouped into low frequency data
%           1 = sum, 0 = average
% add_constant = whether to add a constant in the VAR regression
% ndraws = number of draws in MCMC
% burn_in = number of burn-in draws in MCMC
% -----------------------------------
% Returns:
% Phi_draws = posterior draws of phi(1),,phi(p), d*d*p*R 4-dimension array
% Sigma_draws = posterior draws of covariance matrix, d*d*R 3-dimension array
% constant_draws = posterior draws of the intercept (not the mean), d*R matrix
% -----------------------------------
% Notes:
% Variables may have different frequencies. Each variable may have changing frequencies
% The data Y corresponds to the latent and most frequent data.
% Some disaggregated data are missing (Use NaN), and aggregated or averaged values are observed.
% The program requires aggregated data are recorded at the end of the aggregation interval.
% For example, consider a two-variable VAR,
% where the first variable is sampled quarterly,
% That is, data from Jan. to Mar. are aggregated, from Apr. to Jun. are aggregated.
% Suppose the latent realizations in the six months are 5 9 6 7 3 8
% Then record the data as Y(1,:) = [NaN NaN 20 NaN NaN 18]
% The second variable initially sampled quarterly, but then monthly.
% That is, data from Jan. to Mar. are aggregated, while from Apr. to Jun. are fully observed.
% Suppose the latent realizations in the six months are 2 4 6 5 3 7
% Then record the data as Y(2,:) = [NaN NaN 12 5 3 7]
% The program treats the first value after a run of NaN as the aggregated / averaged data
% 
% Written by Hang Qian, Iowa State University
% Contact me:  matlabist@gmail.com


if nargin < 2;  nlag = 1;   end;
if nargin < 3;  sum_flag = 1;   end;
if nargin < 4;  add_constant = 1;   end;
if nargin < 5;  ndraws = 5000;   end;
if nargin < 6;  burn_in = ndraws * 0.5;   end;

% Analyze data and aggregation structure
[dim,nobs] = size(Y);
if dim > nobs
    Y = Y';
    [dim,nobs] = size(Y);
end
nreg_eq = dim * nlag + add_constant;
nreg_sum = nreg_eq * dim;

miss_ind = isnan(Y);
agg_variable = any(miss_ind,2);
if ~any(agg_variable)
    miss_ind = (Y == 0);
    agg_variable = any(miss_ind,2);
    if any(agg_variable)
        disp('Missing observations not found. Treat 0 as missing value.')
    else
        error('Cannot find any missing observation in the data.')
    end
end

% Determine the aggregation period and the 
% extented period relevant to the missing values in the aggregation period
nagg = find(~miss_ind(find(agg_variable,1),:),1);
period1 = nlag;
period2 = nlag;
nagg_ext = nagg + period1 + period2;
miss_ind_ext = miss_ind(:, 10*nagg+1-period1 : 11*nagg+period2);
miss_ind_ext(:,1:period1) = false;
miss_ind_ext(:,end-period2+1:end) = false;
miss_ind_ext = miss_ind_ext(:);
intact_ind_ext = ~miss_ind_ext;
nmiss = sum(miss_ind_ext);

% Transform summed data into averaged data
if sum_flag == 1
    Y(agg_variable,:) = Y(agg_variable,:) ./ nagg;    
end
    

%----------------------------------------
% Prior distribution settings
%  Beta ~ N(mu,V)
%  Sigma ~ W(Omega,df)
% You may change the hyperparameters here if needed
prior_mu = zeros(nreg_sum,1);
prior_V = 100*eye(nreg_sum);
prior_omega = eye(dim);
prior_df = 10;
%-----------------------------------------

% Preallocate memory
Phi_draws = zeros(dim,dim,nlag,ndraws - burn_in);
Sigma_draws = zeros(dim,dim, ndraws - burn_in);
constant_draws = zeros(dim,ndraws - burn_in);

% Initial draws
Beta_use = prior_mu + chol(prior_V)'*randn(nreg_sum,1);
inv_Sigma = WISHART_RND(prior_omega,prior_df);

Y_sim = Y;
for d = 1:dim
    if agg_variable(d)
        for t = nagg:nagg:nobs
            Y_sim(d,t-nagg+1:t) = Y(d,t);            
        end
    end
end



% For faster codes
X_flag = repmat('X_stack,',1,dim);
X_flag = X_flag(1:end-1);
Phi_big = [zeros(dim,dim*nlag);[eye(dim*(nlag-1)),zeros(dim*(nlag-1),dim)]];
Sigma_big = zeros(dim*nlag,dim*nlag);
update_draws = zeros(sum(agg_variable),nagg-1);


% Construct aggregation transform matrix
A = eye(nagg_ext*dim);
for d = 1:dim
    if agg_variable(d)
        t = period1+nagg;
        loc_A = dim*(t-1)+d;
        A(loc_A,loc_A - dim .* (0:nagg-1)) = 1/nagg;
    end
end


% Gibbs sampler
H_status_bar = waitbar(0,'MCMC in progress    0%');
percentage = floor(ndraws / 100);
for r = 1:ndraws
        
    if mod(r,percentage) == 0
        waitbar(r / ndraws,H_status_bar,['MCMC in progress    ',num2str(floor(r / ndraws * 100)),'%']);
    end
    
    % Config VAR regression data (1/3)
    % Y_SUR is the pooled dependent variable (stack eq. -> stack obs.)
    Y_mat = Y_sim(:, nlag+1:nobs);
    Y_SUR = Y_mat(:);
    
    % Config VAR regression data (2/3)
    % X_stack is the common regressors of every equation
    X_stack = zeros(nobs-nlag,nreg_eq);
    if add_constant == 0
        for p = 1:nlag
            X_stack(:,(p-1)*dim+1:p*dim) = Y_sim(:,nlag-p+1:nobs-p)';
        end
    else
        X_stack(:,1) = ones(nobs-nlag,1);
        for p = 1:nlag
            X_stack(:,(p-1)*dim+2:p*dim+1) = Y_sim(:,nlag-p+1:nobs-p)';
        end
    end
    
    % Config VAR regression data (3/3)
    % X_SUR is the pooled regressors (stack eq. -> stack obs.)
    % but it is easer to stack in the reverse way (stack obs. -> stack eq.)
    % then transform to (stack eq. -> stack obs.)
    X_SUR = eval(['blkdiag(',X_flag,')']);    
    X_temp1 = zeros(nobs-nlag,dim);
    for m = 1:nreg_sum
        X_temp1(:) = X_SUR(:,m);
        X_temp2 = X_temp1';
        X_SUR(:,m) = X_temp2(:);
    end
    
    % Conditional posterior Beta follows N(Dd,D)
    % Normalize data so that the codes runs faster
    P = chol(inv_Sigma);
    Y_mat_sd = P * Y_mat;
    Y_SUR_sd = Y_mat_sd(:);
    X_SUR_sd = zeros(dim*(nobs-nlag),nreg_sum);
    for n = 1:dim
        for s = 1:n
            X_SUR_sd(s:dim:end, (n-1)*nreg_eq+1:n*nreg_eq) = P(s,n) .* X_stack;
        end
    end         
    beta_prec = X_SUR_sd'*X_SUR_sd + inv(prior_V);
    beta_mean = beta_prec \ (X_SUR_sd'*Y_SUR_sd + prior_V \ prior_mu);   
    Beta_use = beta_mean + chol(beta_prec) \ randn(nreg_sum,1);
    
    % Recover Phi and constant from Beta
    Beta_mat = zeros(nreg_eq,dim);
    Beta_mat(:) = Beta_use;
    Beta_mat = Beta_mat';
    if add_constant == 1
        constant_use = Beta_mat(:,1);
        Beta_mat = Beta_mat(:,2:end);
    else
        constant_use = zeros(dim,1);
    end
    Phi_use = zeros(dim,dim,nlag);
    Phi_use(:) = Beta_mat(:);    
    
    % Conditional posterior Sigma follows inverse Wishart
    resid = Y_SUR - X_SUR * Beta_use;
    resid_matrix = zeros(dim,nobs-nlag);
    resid_matrix(:) = resid;    
    RSS_matrix = resid_matrix * resid_matrix';
    post_omega = inv(inv(prior_omega) + RSS_matrix);
    inv_Sigma = WISHART_RND(post_omega,nobs - nlag + prior_df);
    Sigma_use = inv(inv_Sigma);
        
    % Compute VAR(p) covariance structure    
    Phi_big(1:dim,:) = Beta_mat;    
    Sigma_big(1:dim,1:dim) = Sigma_use;
    cov_vec = (eye(dim*nlag*dim*nlag) - kron(Phi_big,Phi_big)) \ Sigma_big(:);
    cov_vec_mat = zeros(dim*nlag,dim*nlag);
    cov_vec_mat(:) = cov_vec;
    
    ACF = zeros(dim,dim,100+nagg_ext-1);
    ACF(:,:,100:100+nlag-1) = reshape(cov_vec_mat(1:dim,:),[dim,dim,nlag]);    
    for m = nlag:nagg_ext-1
        mat_temp = Phi_use(:,:,1) * ACF(:,:, 100+m-1);
        for p = 2:nlag
            mat_temp = mat_temp + Phi_use(:,:,p) * ACF(:,:, 100+m-p);
        end
        ACF(:,:,100+m) = mat_temp;    
    end
    
    % Compute the giant covariance matrix of the nagg_ext periods
    ACF0 = zeros(nagg_ext*dim,dim);
    ACF0(1:dim,:) = tril(ACF(:,:,100)) - diag(0.5 * diag(ACF(:,:,100)));
    for t = 1:nagg_ext-1
        ACF0(dim*t+1:dim*t+dim,:) = ACF(:,:,100+t);
    end
    cov_mat = zeros(nagg_ext*dim,nagg_ext*dim);
    for t = 1:nagg_ext
        cov_mat(dim*(t-1)+1:end, dim*(t-1)+1:dim*(t-1)+dim) = ACF0(1:end-dim*(t-1),:);        
    end
    cov_mat = cov_mat + cov_mat';    
    
    % Transform MVN
    mu_use = (1 - sum(Phi_use,3))\constant_use;
    mu_vec = repmat(mu_use,nagg_ext,1);   
    mu_tran = A * mu_vec;
    cov_tran = A * cov_mat * A';
    mu1 = mu_tran(miss_ind_ext);
    mu2 = mu_tran(intact_ind_ext); 
    Sigma11 = cov_tran(miss_ind_ext,miss_ind_ext);
    Sigma12 = cov_tran(miss_ind_ext,intact_ind_ext);
    Sigma21 = Sigma12';
    Sigma22 = cov_tran(intact_ind_ext,intact_ind_ext);
    ratio = Sigma12 / Sigma22;
    cond_var = Sigma11 - ratio * Sigma21;
    cond_var_chol = chol(cond_var)';
    
    % Sample latent disaggregated data
    % Let t_start = ceil(period1/nagg) * nagg + nagg;
    %     t_end = nobs - ceil(period2/nagg) * nagg;
    % The disaggregated data in [t_start,t_end] is the main body
    % As for the initial [1, t_start-nagg] and the ending [t_end+1:nobs]
    % the sampling is an approximation.    
    for t = nagg : nagg : nobs
        % Sample disaggregated data in the interval [t-nagg+1,t]
        % but need data in [loc1,loc2] to build conditional normal
        loc1 = t - nagg + 1 - period1;
        loc2 = t + period2;
        Y_t = Y(:,t);
        if loc1 < 1
            data_use = [mu_use(:,ones(1 - loc1,1)),Y_sim(:,1:loc2)];
        elseif loc2 > nobs
            data_use = [Y_sim(:,loc1:nobs),mu_use(:,ones(loc2-nobs,1))];  
        else
            data_use = Y_sim(:,loc1:loc2);
        end
        data_use(:,period1+nagg) = Y_t;
        known_var = data_use(intact_ind_ext);               
        cond_mean = mu1 + ratio * (known_var - mu2);
        update_draws(:) = cond_mean + cond_var_chol * randn(nmiss,1);
        Y_sim(agg_variable,t-nagg+1:t-1) = update_draws;
        
        % Lastly sample the degenerated draws               
        Y_sim(agg_variable,t) = nagg .* Y_t(agg_variable) - sum(update_draws,2); 
    end
    
    % Record draws after burn_in periods
    if r > burn_in        
        Phi_draws(:,:,:,r - burn_in) = Phi_use;        
        Sigma_draws(:,:, r - burn_in) = Sigma_use;
        constant_draws(:, r - burn_in) = constant_use;
    end    
end

% Summarize posterior distribution
Phi_mean = mean(Phi_draws,4);
Sigma_mean = mean(Sigma_draws,3);
constant_mean = mean(constant_draws,2);
eval([char([81 72 49 61]),'[87 114 105 116 116 101 110 32 98 121];'])
eval([char([81 72 50 61]),'[32 72 97 110 103 32 81 105 97 110];'])

% Display results
if add_constant == 1
    disp('Constant')
    disp(constant_mean')
    disp(' ')
end

for p = 1:nlag
    disp(['Posterior Phi',num2str(p),' coefficients'])    
    for n = 1:dim
        fprintf('%8.2f', Phi_mean(n,:,p))
        fprintf('\n')
    end
    fprintf('\n')
    disp(' ')
end

disp('Posterior covariance matrix of the VAR system')
for m = 1:dim
    fprintf('%8.2f', Sigma_mean(m,:))
    fprintf('\n')
end
fprintf('\n')
fwrite(1, char([QH1,QH2,10,13]))
close(H_status_bar);

